<?php

namespace App\State\Provider;

use ApiPlatform\Metadata\Get;
use ApiPlatform\Metadata\GetCollection;
use ApiPlatform\Metadata\Operation;
use ApiPlatform\Metadata\Patch;
use ApiPlatform\Metadata\Put;
use ApiPlatform\State\Pagination\TraversablePaginator;
use ApiPlatform\State\ProviderInterface;
use App\Controller\OgRepository\StatusAction;
use App\Dto\Input\ImageInput;
use App\Dto\Input\ImageRepositoryInput;
use App\Dto\Output\ImageOutput;
use App\Dto\Output\ImageRepositoryOutput;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

readonly class ImageRepositoryProvider implements ProviderInterface
{
    public function __construct(
        private ProviderInterface   $collectionProvider,
        private ProviderInterface   $itemProvider,
        private StatusAction        $statusAction
    )
    {
    }

    public function provide(Operation $operation, array $uriVariables = [], array $context = []): object|array|null
    {
        switch ($operation){
            case $operation instanceof GetCollection:
                return $this->provideCollection($operation, $uriVariables, $context);
            case $operation instanceof Patch:
            case $operation instanceof Put:
                return $this->provideInput($operation, $uriVariables, $context);
            case $operation instanceof Get:
                return $this->provideItem($operation, $uriVariables, $context);
        }
    }

    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    private function provideCollection(Operation $operation, array $uriVariables = [], array $context = []): object
    {
        $paginator = $this->collectionProvider->provide($operation, $uriVariables, $context);

        $items = new \ArrayObject();
        foreach ($paginator->getIterator() as $item){
            $statusResponse = $this->statusAction->__invoke($item);
            $content = json_decode($statusResponse->getContent(), true);
            $status = !isset($content['error']);

            $items[] = new ImageRepositoryOutput($item, $status);
        }

        return new TraversablePaginator($items, $paginator->getCurrentPage(), $paginator->getItemsPerPage(), $paginator->getTotalItems());
    }

    public function provideItem(Operation $operation, array $uriVariables = [], array $context = []): object|array|null
    {
        $item = $this->itemProvider->provide($operation, $uriVariables, $context);

        if (!$item) {
            throw new NotFoundHttpException('Image not found');
        }

        return new ImageRepositoryOutput($item);
    }

    public function provideInput(Operation $operation, array $uriVariables = [], array $context = []): object|array|null
    {
        if (isset($uriVariables['uuid'])) {
            $item = $this->itemProvider->provide($operation, $uriVariables, $context);

            return $item !== null ? new ImageRepositoryInput($item) : null;
        }

        return new ImageRepositoryInput();
    }
}
